process.env.NODE_ENV = 'test';

const chai = require('chai');

const dirtyChai = require('dirty-chai');

chai.use(dirtyChai);

const chaiXml = require('chai-xml');

chai.use(chaiXml);

const chaiHttp = require('chai-http');

const assert = chai.assert;

const expect = chai.expect;

const should = chai.should(); // actually call the function

const libs = `${process.cwd()}/libs`;

const server = require(`${libs}/app`);

const mongoose = require('../libs/db/mongoose');
const Simulation = require('../libs/models/simulation');
const User = require('../libs/models/user');

chai.use(chaiHttp);
describe('Requires a simulation', () => {
    let newSimulation;

    beforeEach(() => {
        Simulation.remove({}, (err) => {
            console.log('Test collection purged');
        });
    });

    it('should create a new simulation', (done) => {
        chai.request(server)
            .post('/api/v1/simulation')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({ name: 'test_entry' })
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('id');
                res.body.id.should.be.a('string');
                Simulation.findById(res.body.id, (err, simulation) => {
                    simulation.should.have.property('name');
                    simulation.name.should.be.a('string');
                    simulation.name.should.equal('test_entry');
                    done();
                });
            });
    });
    it('should not create a nameless simulation', (done) => {
        chai.request(server)
            .post('/api/v1/simulation')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.not.have.property('id');
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                Simulation.findById(res.body.id, (err, simulation) => {
                    expect(simulation).to.not.exist;
                    done();
                });
            });
    });
    it('should find an existing simulation', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .get(`/api/v1/simulation/${id}`)
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('_id');
                    res.body._id.should.be.a('string');
                    res.body.should.have.property('name');
                    res.body._id.should.be.a('string');
                    done();
                });
        });
    });
    it('should not find an unexisting simulation', (done) => {
        newSimulation = new Simulation();
        let id = newSimulation._id;
        chai.request(server)
            .get(`/api/v1/simulation/${id}`)
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                done();
            });
    });
    it('should update an existing simulation\'s location', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({ location: 5 })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('id');
                    res.body.id.should.be.a('string');
                    Simulation.findById(res.body.id, (err, simulation) => {
                        simulation.should.have.property('name');
                        simulation.name.should.be.a('string');
                        simulation.name.should.equal('test');
                        simulation.should.have.property('location');
                        simulation.location.should.be.a('number');
                        simulation.location.should.equal(5);
                        done();
                    });
                });
        });
    });
    it('should update multiple fields on a single request', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({
                    name: 'new_name',
                    location: 5,
                    time: 3,
                    failure_rate: [0.1, 0.2, 0.3],
                    goals_care: [0.3, 0.2, 0.1],
                    goals_inclusion: [0.8, 0.9, 1]
                })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('id');
                    res.body.id.should.be.a('string');
                    Simulation.findById(res.body.id, (err, simulation) => {
                        simulation.should.have.property('name');
                        simulation.name.should.be.a('string');
                        simulation.name.should.equal('new_name');
                        simulation.should.have.property('location');
                        simulation.location.should.be.a('number');
                        simulation.location.should.equal(5);
                        simulation.should.have.property('time');
                        simulation.time.should.be.a('number');
                        simulation.time.should.equal(3);
                        simulation.should.have.property('failure_rate');
                        simulation.failure_rate.should.be.a('array');
                        simulation.failure_rate.length.should.equal(3);
                        simulation.should.have.property('goals_care');
                        simulation.goals_care.should.be.a('array');
                        simulation.goals_care.length.should.equal(3);
                        simulation.should.have.property('goals_inclusion');
                        simulation.goals_inclusion.should.be.a('array');
                        simulation.goals_inclusion.length.should.equal(3);
                        done();
                    });
                });
        });
    });
    it('should not update an unexisting simulation', (done) => {
        newSimulation = new Simulation();
        let id = newSimulation._id;
        chai.request(server)
            .post(`/api/v1/simulation/${id}`)
            .send({ location: 5 })
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                done();
            });
    });
    it('should update an existing simulation\'s time', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({ time: 5 })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('id');
                    res.body.id.should.be.a('string');
                    Simulation.findById(res.body.id, (err, simulation) => {
                        simulation.should.have.property('name');
                        simulation.name.should.be.a('string');
                        simulation.should.have.property('time');
                        simulation.time.should.be.a('number');
                        simulation.time.should.equal(5);
                        done();
                    });
                });
        });
    });
    it('should not change results for empty post requests', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(false);
                    done();
                });
        });
    });
    it('should not update in case of invalid field', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({
                    name: 'other_name',
                    totally_not_valid_value_for_an_entry: 'not hacking this api',
                })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(false);
                    Simulation.findById(id, (err, simulation) => {
                        simulation.name.should.equal('test');
                        done();
                    });
                });
        });
    });
    it('should include consistent enrollment tables', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({
                    time: 5,
                    enrollments: "[[100, 150, 200, 250, 300]]",
                })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('id');
                    res.body.id.should.be.a('string');
                    Simulation.findById(res.body.id, (err, simulation) => {
                        simulation.should.have.property('name');
                        simulation.name.should.be.a('string');
                        simulation.should.have.property('time');
                        simulation.time.should.be.a('number');
                        simulation.time.should.equal(5);
                        done();
                    });
                });
        });
    });
    it('should not accept an invalid time', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({
                    time: "I'm an inocent time entry, don't mind me",
                })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(false);
                    });
                    done();
                });
    });
    it('should not accept enrollments table different than provided time', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({
                    time: 5,
                    enrollments: "[[1,2,3]]",
                })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(false);
                    done();
                });
        });
    });
    it('should not include arrays of non arrays as enrollments', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({
                    time: 5,
                    enrollments: "[\"Tomato\"]",
                })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(false);
                    done();
                });
        });
    });
    it('should not accept non array enrollments', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({
                    time: 5,
                    enrollments: "Am I still wanted here?",
                })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(false);
                    done();
                });
        });
    });
    it('should not accept an enrollment with anything other than a number', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({
                    time: 5,
                    enrollments: "[[1,2,\"malicious payload\",4,5]]",
                })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(false);
                    done();
                });
        });
    });
    it('should delete an entry', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .delete(`/api/v1/simulation/${id}`)
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(true);
                    done();
                });
        });
    });
    it('should not delete an unexisting entry', (done) => {
        let sim = new Simulation();
        let id = sim._id;
        chai.request(server)
            .delete(`/api/v1/simulation/${id}`)
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                done();
            });
    });
});

describe('Saves a user', () => {

    beforeEach(() => {
        User.remove({}, (err) => {
            console.log('Test collection purged');
        });
    });

    it('should create a new user', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   password: '123mudar',
                   name: 'Tequila Baby',
                   cpf: '48303270737',
                   schooling: 'Doutorado',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   role: 'Pesquisador',
                   institution_name: 'UFPR',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(true);
                User.findOne({'email': 'lorem@ipsum.com'}, (err, user) => {
                    if (err){
                        console.log('MongoDB error: ' + err);
                    }

                    user.should.have.property('email');
                    done();
                });
            });
    });

    it('should not create a user with an email that is already in use', (done) => {
        let newUser = new User();

        newUser.email = 'lorem@ipsum.com';
        newUser.password = '123mudar';
        newUser.name = 'Gute';
        newUser.cpf = '08236017907';
        newUser.schooling = 'Doutorado';
        newUser.course = 'Ciência da Computação';
        newUser.segment = 'Comunidade acadêmica';
        newUser.role = 'Pesquisador';
        newUser.institution_name = 'UFPR';
        newUser.state = 'PR';
        newUser.city = 'Curitiba';

        newUser.save((err) => {
            if (err) {
                console.log('MongoDB error:' + err);
            }
        }).then(function(newuser){
            chai.request(server)
                .post('/api/v1/user')
                .set('content-type', 'application/x-www-form-urlencoded')
                .set('x-apicache-bypass', 'true')
                .send({email: 'lorem@ipsum.com',
                       password: '123mudar',
                       name: 'Tequila Baby',
                       cpf: '48303270737',
                       schooling: 'Doutorado',
                       course: 'Ciência da Computação',
                       segment: 'Comunidade acadêmica',
                       role: 'Pesquisador',
                       institution_name: 'UFPR',
                       state: 'PR',
                       city: 'Cutiriba'})
                .end((err, res) =>{
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(false);
                    res.body.should.have.property('msg');
                    res.body.msg.should.equal('O email informado já está cadastrado.');
                    User.findOne({'cpf': '48303270737'}, (err, user) => {
                            expect(user).to.not.exist;
                            done();
                    });
                });
        });
    });

    it('should not create a user with a CPF that is already in use', (done) => {
        let newUser = new User();

        newUser.email = 'lorem@ipsum.com';
        newUser.password = '123mudar';
        newUser.name = 'Gute';
        newUser.cpf = '08236017907';
        newUser.schooling = 'Doutorado';
        newUser.course = 'Ciência da Computação';
        newUser.segment = 'Comunidade acadêmica';
        newUser.role = 'Pesquisador';
        newUser.institution_name = 'UFPR';
        newUser.state = 'PR';
        newUser.city = 'Curitiba';

        newUser.save((err) => {
            if (err) {
                console.log('MongoDB error:' + err);
            }
        }).then(function(newuser){
            chai.request(server)
                .post('/api/v1/user')
                .set('content-type', 'application/x-www-form-urlencoded')
                .set('x-apicache-bypass', 'true')
                .send({email: 'dolor@ipsum.com',
                       password: '123mudar',
                       name: 'Tequila Baby',
                       cpf: '08236017907',
                       schooling: 'Doutorado',
                       course: 'Ciência da Computação',
                       segment: 'Comunidade acadêmica',
                       role: 'Pesquisador',
                       institution_name: 'UFPR',
                       state: 'PR',
                       city: 'Cutiriba'})
                .end((err, res) =>{
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('success');
                    res.body.success.should.equal(false);
                    res.body.should.have.property('msg');
                    res.body.msg.should.equal('O CPF informado já está cadastrado.');
                    User.findOne({'email': 'dolor@ipsum.com'}, (err, user) => {
                            expect(user).to.not.exist;
                            done();
                    });
                });
        });
    });

    it('should not save an user without email', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({password: '123mudar',
                   name: 'Tequila Baby',
                   cpf: '48303270737',
                   schooling: 'Doutorado',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   role: 'Pesquisador',
                   institution_name: 'UFPR',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo Email é obrigatório.');
                User.findOne({'cpf': '48303270737'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user without password', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   name: 'Tequila Baby',
                   cpf: '48303270737',
                   schooling: 'Doutorado',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   role: 'Pesquisador',
                   institution_name: 'UFPR',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo Senha é obrigatório.');
                User.findOne({'cpf': '48303270737'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user with invalid email', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'notavalidemail',
                   password: '123mudar',
                   name: 'Tequila Baby',
                   cpf: '48303270737',
                   schooling: 'Doutorado',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   role: 'Pesquisador',
                   institution_name: 'UFPR',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O email informado é inválido.');
                User.findOne({'cpf': '48303270737'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user without name', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   password: '123mudar',
                   cpf: '48303270737',
                   schooling: 'Doutorado',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   role: 'Pesquisador',
                   institution_name: 'UFPR',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo Nome é obrigatório.');
                User.findOne({'cpf': '48303270737'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user without CPF', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   password: '123mudar',
                   name: 'Tequila baby',
                   schooling: 'Doutorado',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   role: 'Pesquisador',
                   institution_name: 'UFPR',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo CPF é obrigatório.');
                User.findOne({'email': 'lorem@ipsum.com'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user without segment', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   password: '123mudar',
                   name: 'Tequila baby',
                   schooling: 'Doutorado',
                   cpf: '48303270737',
                   course: 'Ciência da Computação',
                   role: 'Pesquisador',
                   institution_name: 'UFPR',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo Segmento é obrigatório.');
                User.findOne({'email': 'lorem@ipsum.com'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user without schooling', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   password: '123mudar',
                   name: 'Tequila baby',
                   cpf: '48303270737',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   role: 'Pesquisador',
                   institution_name: 'UFPR',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo Escolaridade é obrigatório.');
                User.findOne({'email': 'lorem@ipsum.com'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user without role', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   password: '123mudar',
                   name: 'Tequila baby',
                   schooling: 'Doutorado',
                   cpf: '48303270737',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   institution_name: 'UFPR',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo Função é obrigatório.');
                User.findOne({'email': 'lorem@ipsum.com'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user without institution', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   password: '123mudar',
                   name: 'Tequila baby',
                   schooling: 'Doutorado',
                   cpf: '48303270737',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   role: 'Pesquisador',
                   state: 'PR',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo Instituição em que trabalha é obrigatório.');
                User.findOne({'email': 'lorem@ipsum.com'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user without city', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   password: '123mudar',
                   name: 'Tequila baby',
                   schooling: 'Doutorado',
                   cpf: '48303270737',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   institution_name: 'UFPR',
                   role: 'Pesquisador',
                   state: 'PR'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo Cidade é obrigatório.');
                User.findOne({'email': 'lorem@ipsum.com'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

    it('should not save an user without state', (done) => {
        chai.request(server)
            .post('/api/v1/user')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({email: 'lorem@ipsum.com',
                   password: '123mudar',
                   name: 'Tequila baby',
                   schooling: 'Doutorado',
                   cpf: '48303270737',
                   course: 'Ciência da Computação',
                   segment: 'Comunidade acadêmica',
                   institution_name: 'UFPR',
                   role: 'Pesquisador',
                   city: 'Cutiriba'})
            .end((err, res) =>{
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('success');
                res.body.success.should.equal(false);
                res.body.should.have.property('msg');
                res.body.msg.should.equal('O campo Estado é obrigatório.');
                User.findOne({'email': 'lorem@ipsum.com'}, (err, user) => {
                        expect(user).to.not.exist;
                        done();
                });
            });
    });

});
